/*
 * Copyright (c) 2003, 2004, 2005 Israfil Consulting Services Corporation
 * Copyright (c) 2003, 2004, 2005 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: DynamicObject.java 23 2006-02-01 17:17:31Z cgruber $
 */
package net.israfil.foundation.core;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * An abstract superclass that implements the Dynamic interface, with 
 * all the necessary lookup logic.  It also implements static equivalents of
 * the Dynamic methods.
 * 
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 */
@Copyright(years={"2003","2004","2005","2006"},owner="Israfil Consulting Services Corporation",license="BSD")
public abstract class DynamicObject implements Dynamic {
	private static Logger logger = Logger.getLogger(DynamicObject.class.getName());

   
    /**
     * @see org.israfil.maveric.Dynamic#perform(java.lang.String, java.lang.Object[])
     */
    public Object perform(String selector, Object ... parameters) {
        return performOn(this,selector,parameters);
    }
    
    /**
     * @see org.israfil.maveric.Dynamic#perform(java.lang.String, java.lang.Object[])
     */
    public static Object performOn(Object receiver, String selector, Object ... parameters) {
    	if (parameters == null) parameters = new Object[]{};
        Method m = getMethodForSelector(receiver,selector);
        try { 
        	if (m != null) return m.invoke(receiver,parameters); 
        } catch (InvocationTargetException ite) {
            logger.log(Level.FINE,ite.getClass().getName()+" thrown attempting to invoke selector: " + selector + " on "+receiver,ite); 
            if (ite.getCause() != null) throw new RuntimeException(ite.getCause());
            else throw new RuntimeException(ite);
        } catch (IllegalAccessException ite) {
            logger.log(Level.FINE,ite.getClass().getName()+" thrown attempting to invoke selector: " + selector + " on "+receiver,ite); 		
            throw new RuntimeException(ite);
        } 
        return null;
    }

    /**
     * @see org.israfil.maveric.Dynamic#respondsTo(java.lang.String)
     */
    public boolean respondsTo(String selector) {
        return respondsTo(this,selector);
    }
    
    /**
     * @see org.israfil.maveric.Dynamic#respondsTo(java.lang.String)
     */
    public static boolean respondsTo(Object receiver, String selector) {
        return (getMethodForSelector(receiver,selector) != null);
    }

    public static Method getMethodForSelector(Object receiver, String selector) {
        if (receiver instanceof Class) 
            return getMethodForSelector((Class)receiver,selector);
        return getMethodForSelector(receiver.getClass(),selector);
    }
    
    public static Method getMethodForSelector(Class receiverClass, String selector)  {
    	try {
    		return _getMethodForSelector(receiverClass,selector);
    	} catch (ClassNotFoundException e) { 
    		throw new RuntimeException(e); 
    	} catch (NoSuchMethodException e) { 
    		return null;
    	} 
    }
    
    static Method _getMethodForSelector(Class receiverClass, String selector) throws ClassNotFoundException, NoSuchMethodException {
        StringTokenizer st = new StringTokenizer(selector,":");
        Class[] paramTypes = new Class[st.countTokens()-1];
        String methodName = st.nextToken();
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        for (int i = 0; st.hasMoreTokens(); i++) {
            String className = st.nextToken();
            if (DynamicUtil._primitiveTypes.containsKey(className)) 
                paramTypes[i] = (Class)DynamicUtil._primitiveTypes.get(className);
            else paramTypes[i] = loader.loadClass(className);
        }
        return receiverClass.getMethod(methodName,paramTypes);
    }

}
