/*
 * Copyright (c) 2003, 2004, 2005 Israfil Consulting Services Corporation
 * Copyright (c) 2003, 2004, 2005 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: DynamicallyAccessibleObject.java 32 2006-02-17 01:40:53Z cgruber $
 */
package net.israfil.foundation.core;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * An implementation of the DynamicallyAccessible interface with the 
 * appropriate infrastructure for retrieving field data, via accessors if
 * such are implemented, or directly from the field where not.
 * 
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 * @author Latest: $Author: cgruber $
 * @version $Revision: 32 $
 */
@Copyright(years={"2003","2004","2005","2006"},owner="Israfil Consulting Services Corporation",license="BSD")
public abstract class DynamicallyAccessibleObject extends DynamicObject implements DynamicallyAccessible {
	private static Logger logger = Logger.getLogger(DynamicallyAccessibleObject.class.getName());
	
	public Object get(String attributeName) {
		try {
			return this._get(attributeName);
		} catch (NoSuchFieldException e) {
			logger.log(Level.FINE,"Object attempted to dynamically access a non-existant or unreadable field.",e);
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			logger.log(Level.FINE,"Object attempted to dynamically access a inaccessible field.",e);
			throw new RuntimeException(e);
		} 
	}

	/**
	 * Internal _get method that throws all the exceptions, to be wrapped by public methods
	 * so that the various exceptions can be given meaning in different contexts (such 
	 * as fetching data, vs. testing for the existance of something.
	 * 
	 * @param attributeName
	 * @return
	 * @throws NoSuchFieldException
	 * @throws IllegalAccessException
	 */
	Object _get(String attributeName) throws NoSuchFieldException, IllegalAccessException {
		if (hasAccessor(attributeName)) return this.perform(getAccessorSelector(attributeName));
		return this.getClass().getField(attributeName).get(this);
	}
	
	/** 
	 * Determines whether an attribute of the given name exists on this object's
	 * definition, or if not, if there is a javabeans-style accessor named 
	 * appropriately for that attribute.  Put another way, determines whether
	 * a get() call would work against this attribute name.
	 * 
	 * The existing implementation is inefficient, but simple to maintain.
	 */
	public boolean hasAttribute(String attributeName) {
		try {
			this._get(attributeName); 
			return true;
		} catch (NoSuchFieldException e) {
			logger.log(Level.FINEST,"Object attempted to dynamically access a non-existant or unreadable field.",e);
			return false;
		} catch (IllegalAccessException e) {
			logger.log(Level.FINEST,"Object attempted to dynamically access a inaccessible field.",e);
			return false;
		}
	}
	
	private static String[] accessPrefixes = new String[] { "get","is" };
	private String getAccessorSelector(String attributeName) {
		if (attributeName == null || attributeName.equals("")) return null;
		String camelAttr = camel(attributeName);
		for (String prefix : accessPrefixes) {
			String selector = prefix+camelAttr;
			if (respondsTo(selector)) return selector;	
		}
		return null;
	}
	
	public boolean hasAccessor(String attributeName) {
		return getAccessorSelector(attributeName) != null;
	}
	
	public static String camel(String string) {
		if (string == null) return null;
		if (string.length() < 1) return string;
		StringBuffer sb = new StringBuffer(string);
		sb.replace(0,1,sb.substring(0,1).toUpperCase());
		return sb.toString();
	}

}
